// =============================================================================
//
//  ConfigTool_MainForm.pas
//
// =============================================================================
//
//   Copyright  2005 by Sascha Willems (www.delphigl.de)
//   Newton Game Dynamics  2003/04/05 by Julio Jerez (www.newtondynamics.com)
//
// =============================================================================
//
//   Contents of this file are subject to the GNU Public License (GPL) which can
//   be obtained here : http://opensource.org/licenses/gpl-license.php
//
// =============================================================================
//
//   Entry for the 2005 PGD "Dog-Fight" Competition
//
// =============================================================================

unit ConfigTool_MainForm;

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, IniFiles, XPMan, SDL, SDLInput, ComCtrls;

type
  TMainForm = class(TForm)
    GroupBox1: TGroupBox;
    DisplayModes: TComboBox;
    Label1: TLabel;
    CheckBoxFullScreen: TCheckBox;
    GroupBox2: TGroupBox;
    CheckBoxSoundEnabled: TCheckBox;
    Button1: TButton;
    Button2: TButton;
    XPManifest1: TXPManifest;
    Button3: TButton;
    GroupBox3: TGroupBox;
    Label5: TLabel;
    ComboController1: TComboBox;
    Label6: TLabel;
    ComboController2: TComboBox;
    GroupBox4: TGroupBox;
    Label7: TLabel;
    EditTCPPort: TEdit;
    GroupBox5: TGroupBox;
    TrackBarDrawDistance: TTrackBar;
    Label3: TLabel;
    Label4: TLabel;
    Label2: TLabel;
    CheckBoxReflections: TCheckBox;
    CheckBoxMusicEnabled: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure CheckBoxFullScreenClick(Sender: TObject);
    procedure CheckBoxSoundEnabledClick(Sender: TObject);
    procedure TrackBarDrawDistanceChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure CheckBoxReflectionsClick(Sender: TObject);
    procedure CheckBoxMusicEnabledClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    procedure EnumerateDisplayModes;
  end;

var
 MainForm     : TMainForm;
 DisplayMode  : array of TDEVMODE;
 SizeX,SizeY  : Word;
 BPP,Freq     : Word;
 DrawDistance : Integer;
 FullScreen   : Boolean;
 Sound        : Boolean;
 Music        : Boolean;
 Reflections  : Boolean;
 Controller1  : String;
 Controller2  : String;
 TCPPort      : Word;

implementation

{$R *.dfm}

// =============================================================================
//  TMainForm.EnumerateDisplayModes
// =============================================================================
procedure TMainForm.EnumerateDisplayModes;
var
 DevMode  : TDevMode;
 ModeNr,i : Integer;
begin
DisplayModes.Items.Clear;
SetLength(DisplayMode, 0);
ModeNr := 0;
while EnumDisplaySettings(nil, ModeNr, DevMode) do
 begin
 if (DevMode.dmBitsPerPel > 8) and (DevMode.dmPelsWidth >= 640) and (DevMode.dmPelsHeight >= 480) then
  begin
  SetLength(DisplayMode, Length(DisplayMode)+1);
  with DisplayMode[High(DisplayMode)] do
   begin
   dmPelsWidth        := DevMode.dmPelsWidth;
   dmPelsHeight       := DevMode.dmPelsHeight;
   dmBitsPerPel       := DevMode.dmBitsPerPel;
   dmDisplayFrequency := DevMode.dmDisplayFrequency;
   end;
  end;
 inc(ModeNr);
 end;
for i := Low(DisplayMode) to High(DisplayMode) do
 with DisplayMode[i] do
  begin
  DisplayModes.Items.Add(IntToStr(dmPelsWidth)+'x'+IntToStr(dmPelsHeight)+' - '+IntToStr(dmBitsPerPel)+' Bit ['+IntToStr(dmDisplayFrequency)+' Hz]');
  if (dmPelsWidth=SizeX) and (dmPelsHeight=SizeY) and (dmBitsPerPel=BPP) and (dmDisplayFrequency=Freq) then
   DisplayModes.ItemIndex := i;
  end;
end;

// =============================================================================
//  TMainForm.FormCreate
// =============================================================================
procedure TMainForm.FormCreate(Sender: TObject);
var
 INI : TIniFile;
 i   : Integer;
begin
Ini          := TIniFile.Create(ExtractFilePath(Application.ExeName)+'\settings.ini');
SizeX        := Ini.ReadInteger('video', 'resx', 800);
SizeY        := Ini.ReadInteger('video', 'resy', 600);
Freq         := Ini.ReadInteger('video', 'frequency', 60);
DrawDistance := Ini.ReadInteger('video', 'drawdistance', 1024);
Reflections  := Ini.ReadBool('video', 'reflections', False);
FullScreen   := Ini.ReadBool('video', 'fullscreen', False);
Sound        := Ini.ReadBool('sound', 'enabled', True);
Music        := Ini.ReadBool('music', 'enabled', True);
Controller1  := Ini.ReadString('input', 'player1', 'keyboard');
Controller2  := Ini.ReadString('input', 'player2', '0');
TCPPort      := Ini.ReadInteger('network', 'port', 5000);
BPP          := 32;
Ini.Free;
EnumerateDisplayModes;
EditTCPPort.Text              := IntToStr(TCPPort);
CheckBoxFullScreen.Checked    := FullScreen;
CheckBoxSoundEnabled.Checked  := Sound;
CheckBoxMusicEnabled.Checked  := Music;
CheckBoxReflections.Checked   := Reflections;
TrackBarDrawDistance.Position := DrawDistance;
// Find controllers and add to drop downs
ComboController1.AddItem('Keyboard (Cursorkeys)', nil);
ComboController2.AddItem('Keyboard (Cursorkeys)', nil);
SDL_Init(SDL_INIT_JOYSTICK);
if SDL_NumJoysticks > 0 then
 for i := 0 to SDL_NumJoysticks-1 do
  begin
  ComboController1.AddItem(SDL_JoystickName(i)+'[ID:'+IntToStr(i)+']', nil);
  ComboController2.AddItem(SDL_JoystickName(i)+'[ID:'+IntToStr(i)+']', nil);
  end;
SDL_Quit;
if Controller1 = 'keyboard' then
 ComboController1.ItemIndex := 0
else
 ComboController1.ItemIndex := StrToInt(Controller1)+1;
if Controller2 = 'keyboard' then
 ComboController2.ItemIndex := 0
else
 ComboController2.ItemIndex := StrToInt(Controller2)+1;
end;

// =============================================================================
//  TMainForm.FormClose
// =============================================================================
procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
end;

// =============================================================================
//  TMainForm.CheckBoxFullScreenClick
// =============================================================================
procedure TMainForm.CheckBoxFullScreenClick(Sender: TObject);
begin
FullScreen := CheckBoxFullScreen.Checked;
end;

// =============================================================================
//  TMainForm.CheckBoxSoundEnabledClick
// =============================================================================
procedure TMainForm.CheckBoxSoundEnabledClick(Sender: TObject);
begin
Sound := CheckBoxSoundEnabled.Checked;
end;

// =============================================================================
//  TMainForm.TrackBarDrawDistanceChange
// =============================================================================
procedure TMainForm.TrackBarDrawDistanceChange(Sender: TObject);
begin
DrawDistance := TrackBarDrawDistance.Position;
end;

// =============================================================================
//  TMainForm.Button1Click
// =============================================================================
procedure TMainForm.Button1Click(Sender: TObject);
var
 INI : TIniFile;
begin
Ini := TIniFile.Create(ExtractFilePath(Application.ExeName)+'\settings.ini');
SizeX   := DisplayMode[DisplayModes.ItemIndex].dmPelsWidth;
SizeY   := DisplayMode[DisplayModes.ItemIndex].dmPelsHeight;
Freq    := DisplayMode[DisplayModes.ItemIndex].dmDisplayFrequency;
TCPPort := StrToInt(EditTCPPort.Text);
Ini.WriteInteger('video', 'resx', SizeX);
Ini.WriteInteger('video', 'resy', SizeY);
Ini.WriteInteger('video', 'frequency', Freq);
Ini.WriteInteger('video', 'drawdistance', DrawDistance);
Ini.WriteBool('video', 'reflections', Reflections);
Ini.WriteBool('video', 'fullscreen', FullScreen);
Ini.WriteBool('sound', 'enabled', Sound);
Ini.WriteBool('music', 'enabled', Music);
Ini.WriteInteger('network', 'port', TCPPort);
if ComboController1.ItemIndex = 0 then
 Ini.WriteString('input', 'player1', 'keyboard')
else
 Ini.WriteString('input', 'player1', IntToStr(ComboController1.ItemIndex-1));
if ComboController2.ItemIndex = 0 then
 Ini.WriteString('input', 'player2', 'keyboard')
else
 Ini.WriteString('input', 'player2', IntToStr(ComboController2.ItemIndex-1));
Ini.Free;
Close;
end;

// =============================================================================
//  TMainForm.Button2Click
// =============================================================================
procedure TMainForm.Button2Click(Sender: TObject);
begin
SizeX        := 800;
SizeY        := 600;
Freq         := 60;
DrawDistance := 1024;
FullScreen   := False;
Sound        := True;
TCPPort      := 5000;
EnumerateDisplayModes;
EditTCPPort.Text              := IntToStr(TCPPort);
CheckBoxFullScreen.Checked    := FullScreen;
CheckBoxSoundEnabled.Checked  := Sound;
TrackBarDrawDistance.Position := DrawDistance;
end;

// =============================================================================
//  TMainForm.Button3Click
// =============================================================================
procedure TMainForm.Button3Click(Sender: TObject);
begin
Close;
end;

// =============================================================================
//  TMainForm.CheckBoxReflectionsClick
// =============================================================================
procedure TMainForm.CheckBoxReflectionsClick(Sender: TObject);
begin
Reflections := CheckBoxReflections.Checked;
end;

// =============================================================================
//  TMainForm.CheckBoxReflectionsClick
// =============================================================================
procedure TMainForm.CheckBoxMusicEnabledClick(Sender: TObject);
begin
Music := CheckBoxMusicEnabled.Checked;
end;

end.
