// =============================================================================
//
//  JagdGeschwader_MainForm.pas
//
// =============================================================================
//
//   Copyright  2005 by Sascha Willems (www.delphigl.de)
//
// =============================================================================
//
//   Contents of this file are subject to the GNU Public License (GPL) which can
//   be obtained here : http://opensource.org/licenses/gpl-license.php
//
// =============================================================================
//
//   Entry for the 2005 PGD "Dog-Fight" Competition
//
// =============================================================================

unit JagdGeschwader_MainForm;

interface

uses
  SDL,
  SDLInput,

  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ExtCtrls,
  IniFiles,

  dglOpenGL,
  NewtonImport,
  glMatrixHelper,
  gl3DS,

  JagdGeschwader_Effects,
  JagdGeschwader_SoundSystem,
  JagdGeschwader_AirCraft,
  JagdGeschwader_GameClass,
  JagdGeschwader_FirstStartForm;

type
  TGLForm = class(TForm)
    InitTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure InitTimerTimer(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private-Deklarationen }
  public
    RC : HGLRC;
    DC : HDC;
    procedure GoToFullScreen(pWidth, pHeight, pBPP, pFrequency : Word);
    procedure LoadSettings;
  end;

var
 GLForm      : TGLForm;
 NewtonWorld : PNewtonWorld;
 MousePos    : TPoint;

implementation

{$R *.dfm}
// *****************************************************************************
//  TGLForm
// *****************************************************************************

// =============================================================================
//  TGLForm.GoToFullScreen
// =============================================================================
procedure TGLForm.GoToFullScreen(pWidth, pHeight, pBPP, pFrequency : Word);
var
 dmScreenSettings : DevMode;
begin
WindowState := wsMaximized;
BorderStyle := bsNone;
ZeroMemory(@dmScreenSettings, SizeOf(dmScreenSettings));
with dmScreenSettings do
 begin
 dmSize              := SizeOf(dmScreenSettings);
 dmPelsWidth         := pWidth;
 dmPelsHeight        := pHeight;
 dmBitsPerPel        := pBPP;
 dmDisplayFrequency  := pFrequency;
 dmFields            := DM_PELSWIDTH or DM_PELSHEIGHT or DM_BITSPERPEL or DM_DISPLAYFREQUENCY;
 end;
if (ChangeDisplaySettings(dmScreenSettings, CDS_FULLSCREEN) = DISP_CHANGE_FAILED) then
 begin
 MessageBox(0, 'Could not switch to desired fullscreen mode!', 'Error', MB_OK or MB_ICONERROR);
 exit
 end;
end;

// =============================================================================
//  TGLForm.LoadSettings
// =============================================================================
//  Load game settings from INI-file
// =============================================================================
procedure TGLForm.LoadSettings;
var
 INI : TIniFile;
begin
INI := TIniFile.Create(ExtractFilePath(Application.ExeName)+'settings.ini');
Settings.Resolution.X := INI.ReadInteger('video', 'resx', 640);
Settings.Resolution.Y := INI.ReadInteger('video', 'resy', 480);
Settings.Frequency    := INI.ReadInteger('video', 'frequency', 60);
Settings.DrawDistance := INI.ReadInteger('video', 'drawdistance', 1024);
Settings.Reflections  := INI.ReadBool('video', 'reflections', False);
Settings.Fullscreen   := INI.ReadBool('video', 'fullscreen', False);
Settings.Sound        := INI.ReadBool('sound', 'enabled', True);
Settings.Music        := INI.ReadBool('music', 'enabled', False);
Settings.Controller1  := INI.ReadString('input', 'player1', 'keyboard');
Settings.Controller2  := INI.ReadString('input', 'player2', '0');
Settings.TCPPort      := INI.ReadInteger('network', 'port', 5000);
INI.Free;
end;

// =============================================================================
//  TGLForm.FormCreate
// =============================================================================
procedure TGLForm.FormCreate(Sender: TObject);
begin
ChDir(ExtractFilePath(Application.ExeName));
Log.Add(TimeToStr(Now)+': Logging started...');
LoadSettings;
if Settings.Fullscreen then
 begin
 Log.Add(TimeToStr(Now)+': Running in fullscreen mode');
 GoToFullscreen(Settings.Resolution.X, Settings.Resolution.Y, 32, Settings.Frequency);
 BorderStyle := bsNone;
 WindowState := wsMaximized;
 end
else
 begin
 Log.Add(TimeToStr(Now)+': Running in windowed mode');
// BorderStyle  := bsNone;
 ClientWidth  := Settings.Resolution.X;
 ClientHeight := Settings.Resolution.Y;
 end;
Cursor := crNone;
// SDL =========================================================================
Log.Add(TimeToStr(Now)+': Initializing SDL...');
if SDL_Init(SDL_INIT_JOYSTICK) < 0 then
 begin
 Log.Add(TimeToStr(Now)+':  Fatal error : Could not initialize SDL');
 raise Exception.Create('Could not initialize SDL!');
 Close;
 end;
Log.Add(TimeToStr(Now)+':  SDL_Joystick initialized');

// OpenGL ======================================================================
Log.Add(TimeToStr(Now)+': Initialzing OpenGL...');
InitOpenGL;
DC := GetDC(Handle);
RC := CreateRenderingContext(DC, [opDoubleBuffered], 32, 24, 0, 0, 0, 0);
ActivateRenderingContext(DC, RC);
glEnable(GL_DEPTH_TEST);
glDepthFunc(GL_LESS);
glClearColor(0,0,0.3,0);
Log.Add(TimeToStr(Now)+': Done');

// Newton ======================================================================
NewtonWorld := NewtonCreate(nil, nil);

InitTimer.Enabled := True;
end;

// =============================================================================
//  TGLForm.InitTimerTimer
// =============================================================================
procedure TGLForm.InitTimerTimer(Sender: TObject);
var
 TmpMenu : TSubMenu;
 i       : Integer;
begin
InitTimer.Enabled := False;

if SDL_NumJoysticks > 0 then
 for i := 0 to SDL_NumJoysticks-1 do
  begin
  SDLJoystick := SDL_JoystickOpen(i);
  Log.Add(TimeToStr(Now)+': Found joystick "'+SDL_JoystickName(i)+'"');
  Log.Add(TimeToStr(Now)+':  Number of axes     = "'+IntToStr(SDL_JoystickNumAxes(SDLJoystick))+'"');
  Log.Add(TimeToStr(Now)+':  Number of buttons  = "'+IntToStr(SDL_JoystickNumButtons(SDLJoystick))+'"');
  Log.Add(TimeToStr(Now)+': Done');
  SDL_JoystickClose(SDLJoyStick);
  end;

GameClass := TGameClass.Create(self, NewtonWorld, DC);
with GameClass do
 begin
 StartTick := GetTickCount;
 AddTexturesInDir('data\special\', False);
 UpdateLoadingScreen('Loading menu backdrops');
 AddTexturesInDir('data\menu\', False);
 UpdateLoadingScreen('Loading textures');
 AddTexturesInDir('data\textures\');
 UpdateLoadingScreen('Loading previews images');
 LoadPreviewImages;
 LensFlare.SetTextures(BindTexture('halo'), BindTexture('bigglow'), BindTexture('glow'), BindTexture('streak'));
 CrateModel := TAll3DSMesh.Create(nil);
 CrateModel.TexturePath := 'data\textures\3ds\';
 CrateModel.LoadFromFile('data\crate1.3ds');
 CrateModel.Mesh[0].BuildDisplayList;
 end;

if NeedSerializations then
 begin
 Application.CreateForm(TFirstStartForm, FirstStartForm);
 FirstStartForm.StartTimer.Enabled := True;
 FirstStartForm.ShowModal;
 FirstStartForm.Free;
 end;

// Build game menu and menu structure
// You normally wouldn't do that in code but rather with e.g. an editor or an
// XML file. But the menu of this game is rather simple so we hardcode it.
GameMenu := TGameMenu.Create(self, 'data\font1.tga', DC);
with GameMenu do
 begin
 TmpMenu  := AddSubMenu('mainmenu', 'mainmenu');
 AddMenuItem(TmpMenu, 'mmbtn1', 'menu_newgame',     SPoint(199,204),    SPoint(200,200));
 AddMenuItem(TmpMenu, 'mmbtn2', 'menu_startserver', SPoint(302,345),    SPoint(200,200));
 AddMenuItem(TmpMenu, 'mmbtn3', 'menu_startclient', SPoint(408,483),    SPoint(200,200));
 AddMenuItem(TmpMenu, 'mmbtn4', 'menu_credits',     SPoint(700,403),    SPoint(128,128));
 AddMenuItem(TmpMenu, 'mmbtn5', 'quit',             SPoint(699,508),    SPoint(128,128));
 TmpMenu  := AddSubMenu('credits', 'credits');
 AddMenuItem(TmpMenu, 'credbtn1', 'menu_mainmenu',  SPoint(695, 514),   SPoint(128,128));
 TmpMenu  := AddSubMenu('newgame', 'newgame');
 AddMenuItem(TmpMenu, 'ngbtn1', 'menu_mainmenu',    SPoint(685, 373),   SPoint(128,128));
 AddMenuItem(TmpMenu, 'ngbtn2', 'startgame',        SPoint(687, 505),   SPoint(200,200));
 AddMenuItem(TmpMenu, 'ngbtn3', 'prevaircraft_p1',  SPoint(71, 166.7),  SPoint(64,64));
 AddMenuItem(TmpMenu, 'ngbtn4', 'nextaircraft_p1',  SPoint(71, 215.3),  SPoint(64,64));
 AddMenuItem(TmpMenu, 'ngbtn3', 'prevaircraft_p2',  SPoint(723, 166.7), SPoint(64,64));
 AddMenuItem(TmpMenu, 'ngbtn4', 'nextaircraft_p2',  SPoint(723, 215.3), SPoint(64,64));
 AddMenuItem(TmpMenu, 'ngbtn5', 'prevscenario',     SPoint(260, 532.7), SPoint(80,80));
 AddMenuItem(TmpMenu, 'ngbtn6', 'nextscenario',     SPoint(324, 532.7), SPoint(80,80));
 TmpMenu  := AddSubMenu('gameover', 'gameover');
 AddMenuItem(TmpMenu, 'credbtn1', 'menu_mainmenu',  SPoint(695, 514),   SPoint(128,128));
 Visible := True;
 end;
SoundSystem := TSoundSystem.Create(self, not Settings.Sound);
SoundSystem.OpenBackgroundStream('data\music\song1.mp3');
if Settings.Music then
 SoundSystem.PlayBackgroundStream;
SoundSystem.AddSamplesInDir('data\sounds', '*.mp3');
SoundSystem.AddSamplesInDir('data\sounds', '*.wav');
GameClass.GameLoop(DC);
end;

// =============================================================================
//  TGLForm.FormDestroy
// =============================================================================
procedure TGLForm.FormDestroy(Sender: TObject);
begin
SDL_JoystickClose(SDLJoystick); // NOTE : Sometimes crashes the app on exit, dunno why
SDL_Quit;
if Assigned(GameClass) then
 GameClass.Free;
if Assigned(NewtonWorld) then
 NewtonDestroy(NewtonWorld);
DeactivateRenderingContext;
wglDeleteContext(RC);
ReleaseDC(Handle, DC);
if Settings.Fullscreen then
 ChangeDisplaySettings(devmode(nil^), 0);
end;

// =============================================================================
//  TGLForm.FormKeyPress
// =============================================================================
procedure TGLForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
case Key of
 #27 : GameClass.EndGame;
 'd' : ShowDebug         := not ShowDebug;
 'p' : GameClass.Pause   := not GameClass.Pause;
 'f' : GameClass.ShowFPS := not GameClass.ShowFPS;
 // DONE : For debug purposes only, remove for final build
 //'s' : GameClass.SplitScreen := not GameClass.SplitScreen;
end;
end;

// =============================================================================
//  TGLForm.FormMouseMove
// =============================================================================
procedure TGLForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
 MPos : TPoint;
begin
if (ssRight in Shift) then
 begin
 MPos := Mouse.CursorPos;
 GameClass.SceneRotation[0] := GameClass.SceneRotation[0]+(MPos.Y-MousePos.Y)*0.25;
 GameClass.SceneRotation[1] := GameClass.SceneRotation[1]+(MPos.X-MousePos.X)*0.25;
 MousePos := MPos;
 if MPos.X < Left+15 then
  SetCursorPos(Left+ClientWidth-17, MPos.Y);
 if MPos.X > Left+ClientWidth-15 then
  SetCursorPos(Left+17, MPos.Y);
 if MPos.Y < Top+35 then
  SetCursorPos(MPos.X, Top+ClientHeight-17);
 if MPos.Y > Top+ClientHeight-15 then
  SetCursorPos(MPos.X, Top+37);
 end;
MousePos := Mouse.CursorPos;
if Assigned(GameMenu) then
 GameMenu.MousePos := Point(Round(X/ClientWidth*800), Round(Y/ClientHeight*600));
end;

// =============================================================================
//  TGLForm.FormClick
// =============================================================================
procedure TGLForm.FormClick(Sender: TObject);
begin
GameMenu.ProcessInput;
end;

// =============================================================================
//  TGLForm.FormKeyDown
// =============================================================================
procedure TGLForm.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
if not GameMenu.Visible then
 case Key of
  vk_F1 : GameClass.AirCraft[0].Camera := ctCloseBehind;
  vk_F2 : GameClass.AirCraft[0].Camera := ctFarBehind;
  vk_F3 : GameClass.AirCraft[0].Camera := ctAboveCockpit;
  vk_F4 : GameClass.AirCraft[0].Camera := ctFreeRotate;
  vk_F5 : GameClass.AirCraft[1].Camera := ctCloseBehind;
  vk_F6 : GameClass.AirCraft[1].Camera := ctFarBehind;
  vk_F7 : GameClass.AirCraft[1].Camera := ctAboveCockpit;
  vk_F8 : GameClass.AirCraft[1].Camera := ctFreeRotate;
  vk_F9 : if SoundSystem.Streaming then
           SoundSystem.StopBackgroundStream
          else
           SoundSystem.PlayBackgroundStream;
 end;
end;

// =============================================================================
//  TGLForm.NetworkTimerTimer
// =============================================================================
procedure TGLForm.FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
GameClass.SceneZoom := GameClass.SceneZoom + 0.025 * WheelDelta;
end;

// =============================================================================
//  TGLForm.FormClose
// =============================================================================
procedure TGLForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
GameClass.Quit := True;
end;

end.
