// =============================================================================
//
//  glFont.pas
//   Simple class for OpenGL texture fonts
//
// =============================================================================
//
//   Copyright  2003-2005 by Sascha Willems (www.delphigl.de)
//
// =============================================================================
//
//   Contents of this file are subject to the GNU Public License (GPL) which can
//   be obtained here : http://opensource.org/licenses/gpl-license.php
//
// =============================================================================

unit glFont;

interface

uses
 Windows,
 SysUtils,
 dglOpenGL,
 Textures;

type
 TTexFont = class
   FontWidth   : array[0..255] of Single;
   FontBase    : Cardinal;
   FontTexture : glUInt;
   function GetWidth(pText : String) : Single;
   constructor Create(pFontTextureFile, pFontSizeFile : String;pSizeDivider : Single); overload;
   destructor Destroy; override;
   procedure Print3D(pText : String;pPosition : array of Single;pScaleFactor : Single);
   procedure Print2D(pText : String;pPosition : array of Single;pScaleFactor : Single;pShadow : Boolean = False); overload;
   procedure Print2DCenter(pText : String;pPosition : array of Single;pScaleFactor : Single;pShadow : Boolean = False); overload;
   procedure Print2D(pText : String;pPosition,pColor : array of Single;pAlpha, pScaleFactor : Single;pShadow : Boolean = False); overload;
   procedure Print2DEx(pText : String;pPosition,pColor : array of Single;pAlpha, pScaleFactor : Single;pCenter : Boolean);
  end;

var
 TextureFont  : TTexFont;

implementation

// *****************************************************************************
//  TTexFont
// *****************************************************************************

// =============================================================================
//  TTexFont.Create                                              - CONSTRUCTOR -
// =============================================================================
constructor TTexFont.Create(pFontTextureFile, pFontSizeFile : String;pSizeDivider : Single);
var
 F      : File of Byte;
 i,B    : Byte;
 x,y    : Single;
 xs     : Single;
begin
inherited Create;
LoadTexture(pFontTextureFile, FontTexture);
if not FileExists(pFontSizeFile) then
 begin
 for i := 0 to 255 do
  FontWidth[i] := 16;
 end
else
 begin
 AssignFile(F, pFontSizeFile);
 Reset(F);
 for i := 0 to 255 do
  begin
  Read(F, B);
  FontWidth[i] := B;
  FontWidth[i] := FontWidth[i] / pSizeDivider;
  end;
 CloseFile(F);
 end;
FontBase := glGenLists(256);
for i := 0 to 255 do
 begin
 x := (i mod 16) / 16;
 y := (i div 16) / 16;
 glNewList(FontBase+i, GL_COMPILE);
  glBegin(GL_QUADS);
   xs := (16-FontWidth[I])/2/256;
   glTexCoord2f(x+xs,                            1-y);        glVertex2f(0,            0);
   glTexCoord2f(x+xs + FontWidth[I]/256 + 1/512, 1-y);        glVertex2f(FontWidth[I], 0);
   glTexCoord2f(x+xs + FontWidth[I]/256 + 1/512, 1-y-0.0625); glVertex2f(FontWidth[I], 16);
   glTexCoord2f(x+xs,                            1-y-0.0625); glVertex2f(0,            16);
  glEnd;
  glTranslatef(FontWidth[i]+1, 0, 0);
 glEndList;
 end;
end;

// =============================================================================
//  TTexFont.Destroy                                              - DESTRUCTOR -
// =============================================================================
destructor TTexFont.Destroy;
begin
inherited;
glDeleteLists(FontBase, 256);
end;

// =============================================================================
//  TTexFont.Print3D
// =============================================================================
procedure TTexFont.Print3D(pText : String;pPosition : array of Single;pScaleFactor : Single);
begin
glBindTexture(GL_TEXTURE_2D, FontTexture);
glPushMatrix;
 glTranslatef(pPosition[0],pPosition[1],pPosition[2]);
 glScalef(pScaleFactor, -pScaleFactor , pScaleFactor);
 glListBase(FontBase);
 glCallLists(Length(pText), GL_UNSIGNED_BYTE, PChar(pText));
glPopMatrix;
glColor4f(1,1,1,1);
glListBase(0);
end;

// =============================================================================
//  TTexFont.GetWidth
// =============================================================================
function TTexFont.GetWidth(pText : String) : Single;
var
 i : Integer;
begin
Result := 0;
if Length(pText) > 0 then
 for i := 1 to Length(pText) do
  Result := Result + FontWidth[Ord(pText[i])]+1;
end;

// =============================================================================
//  TTexFont.Print2D
// =============================================================================
procedure TTexFont.Print2D(pText : String;pPosition : array of Single;pScaleFactor : Single;pShadow : Boolean = False);
begin
if Length(pText) = 0 then
 exit;
glBindTexture(GL_TEXTURE_2D, FontTexture);
glEnable(GL_BLEND);
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
glColor3f(1,1,1);
glPushMatrix;
 glTranslatef(pPosition[0],pPosition[1],pPosition[2]);
 glScalef(pScaleFactor, pScaleFactor , pScaleFactor);
 glListBase(FontBase);
 if pShadow then
  begin
  glDepthMask(False);
  glColor3f(0,0,0);
  glPushMatrix;
   glTranslatef(1.5*pScaleFactor, 1.5*pScaleFactor, 0);
   glCallLists(Length(pText), GL_UNSIGNED_BYTE, PChar(pText));
  glPopMatrix;
  glColor3f(1,1,1);
  glDepthMask(True);
  end;
 glCallLists(Length(pText), GL_UNSIGNED_BYTE, PChar(pText));
glPopMatrix;
glColor4f(1,1,1,1);
glDisable(GL_BLEND);
glListBase(0);
end;

// =============================================================================
//  TTexFont.Print2D
// =============================================================================
procedure TTexFont.Print2DCenter(pText : String;pPosition : array of Single;pScaleFactor : Single;pShadow : Boolean = False);
begin
glPushMatrix;
 glTranslatef(-GetWidth(pText)*pScaleFactor / 2, 0, 0);
 Print2D(pText, pPosition, pScaleFactor, pShadow);
glPopMatrix;
end;

// =============================================================================
//  TTexFont.Print2D
// =============================================================================
procedure TTexFont.Print2D(pText : String;pPosition,pColor : array of Single;pAlpha, pScaleFactor  : Single;pShadow : Boolean = False);
begin
if Length(pText) = 0 then
 exit;
glBindTexture(GL_TEXTURE_2D, FontTexture);
//if pAlpha < 1 then
 begin
 glEnable(GL_BLEND);
 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 end;
glColor4f(pColor[0], pColor[1], pColor[2], pAlpha);
glPushMatrix;
 glTranslatef(pPosition[0],pPosition[1],pPosition[2]);
 glScalef(pScaleFactor, pScaleFactor , pScaleFactor);
 glListBase(FontBase);
 if pShadow then
  begin
  glDepthMask(False);
  glColor4f(0,0,0,0.5);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glPushMatrix;
   glTranslatef(1.5*pScaleFactor, 1.5*pScaleFactor, 0);
   glCallLists(Length(pText), GL_UNSIGNED_BYTE, PChar(pText));
  glPopMatrix;
  glColor3f(1,1,1);
  glDepthMask(True);
  end;
 glCallLists(Length(pText), GL_UNSIGNED_BYTE, PChar(pText));
glPopMatrix;
glColor4f(1,1,1,1);
if pAlpha < 1 then
 glDisable(GL_BLEND);
glListBase(0);
end;

// =============================================================================
//  TTexFont.Print2DEx
// =============================================================================
procedure TTexFont.Print2DEx(pText : String;pPosition,pColor : array of Single;pAlpha, pScaleFactor : Single;pCenter : Boolean);
begin
glDisable(GL_DEPTH_TEST);
glBindTexture(GL_TEXTURE_2D, FontTexture);
if pAlpha < 1 then
 begin
 glEnable(GL_BLEND);
 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 end;
glColor4f(pColor[0], pColor[1], pColor[2], pAlpha);
glPushMatrix;
 if pCenter then
  pPosition[0] := pPosition[0]-Length(pText)*6*pScaleFactor;
 glTranslatef(pPosition[0],pPosition[1],pPosition[2]);
 glScalef(pScaleFactor, pScaleFactor , pScaleFactor);
 glListBase(FontBase);
 glCallLists(Length(pText), GL_UNSIGNED_BYTE, Pointer(pText));
glPopMatrix;
glColor4f(1,1,1,1);
glEnable(GL_DEPTH_TEST);
if pAlpha < 1 then
 glDisable(GL_BLEND);
glListBase(0);
end;

end.

